"""
mpcABX - by Marco Pontello
Simple Python script to use a device with MPD as an ABX comparator.
Manage just MPD control & selection of which track to play.
Another MPD client is needed to setup the playlist.
Just the first two tracks will be used.
Inspired by foobar2000 ABX Comparator component.
"""

import telnetlib
import random
import sys
import logging

def guessprob(trials, correct):
    # data from: http://home.provide.net/~djcarlst/abx_bino.htm (1/26/2011)
    #            Probability of Experimental Result Being the Same as Random Guesses
    #            by David Carlstrom - 
    probtable = [
    [0.5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
    [0.750, 0.250, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
    [0.875, 0.500, 0.125, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
    [0.938, 0.688, 0.312, 0.063, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
    [0.969, 0.813, 0.500, 0.188, 0.031, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
    [0.984, 0.891, 0.656, 0.344, 0.109, 0.016, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
    [0.992, 0.938, 0.773, 0.500, 0.227, 0.062, 0.008, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 
    [0.996, 0.965, 0.855, 0.637, 0.363, 0.145, 0.035, 0.004, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
    [0.998, 0.980, 0.910, 0.746, 0.500, 0.254, 0.090, 0.020, 0.002, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
    [0.999, 0.989, 0.945, 0.828, 0.623, 0.377, 0.172, 0.055, 0.011, 0.001, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 
    [1.000, 0.994, 0.967, 0.887, 0.726, 0.500, 0.274, 0.113, 0.033, 0.006, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],  
    [1.000, 0.997, 0.981, 0.927, 0.806, 0.613, 0.387, 0.194, 0.073, 0.019, 0.003, 0, 0, 0, 0, 0, 0, 0, 0, 0],
    [1.000, 0.998, 0.989, 0.954, 0.867, 0.709, 0.500, 0.291, 0.133, 0.046, 0.011, 0.002, 0, 0, 0, 0, 0, 0, 0, 0],
    [1.000, 0.999, 0.994, 0.971, 0.910, 0.788, 0.605, 0.395, 0.212, 0.090, 0.029, 0.006, 0.001, 0, 0, 0, 0, 0, 0, 0],
    [1.000, 1.000, 0.996, 0.982, 0.941, 0.849, 0.696, 0.500, 0.304, 0.151, 0.059, 0.018, 0.004, 0, 0, 0, 0, 0, 0, 0],
    [1.000, 1.000, 0.998, 0.989, 0.962, 0.895, 0.773, 0.598, 0.402, 0.227, 0.105, 0.038, 0.011, 0.002, 0, 0, 0, 0, 0, 0],
    [1.000, 1.000, 0.999, 0.994, 0.975, 0.928, 0.834, 0.685, 0.500, 0.315, 0.166, 0.072, 0.025, 0.006, 0.001, 0, 0, 0, 0, 0],
    [1.000, 1.000, 0.999, 0.996, 0.985, 0.952, 0.881, 0.760, 0.593, 0.407, 0.240, 0.119, 0.048, 0.015, 0.004, 0.001, 0, 0, 0, 0],
    [1.000, 1.000, 1.000, 0.998, 0.990, 0.968, 0.916, 0.820, 0.676, 0.500, 0.324, 0.180, 0.084, 0.032, 0.010, 0.002, 0, 0, 0, 0],
    [1.000, 1.000, 1.000, 0.999, 0.994, 0.979, 0.942, 0.868, 0.748, 0.588, 0.412, 0.252, 0.132, 0.058, 0.021, 0.006, 0.001, 0, 0, 0]
    ]
    return probtable[trials][correct] * 100


def abx(mpdaddr, mpdport, trialsnum, logfile):
    print "Connecting with MPD..."
    tn = telnetlib.Telnet(mpdaddr, mpdport)
    print "OK"
    random.seed()
    startpos = 0
    correctnum = 0
    for trial in range(0, trialsnum):
        trackx = random.randint(0, 1)
        logging.info("trial %i/%i" % (trial+1, trialsnum)) 
        while True:
            print "trial %i of %i" % (trial+1, trialsnum)
            print "  play menu:"
            print "    a: play track A"
            print "    b: play track B"
            print "    x: play track X"
            print "    s: set start position"
            print "    r: ready to answer"
            cmd = raw_input("    ? ")
            if cmd == "a":
                tn.write("play 0\nseek 0 %i\n" % startpos)
            elif cmd == "b":
                tn.write("play 1\nseek 1 %i\n" % startpos)
            elif cmd == "x":
                tn.write("play %i\nseek %i %i\n" % (trackx, trackx, startpos))
            elif cmd == "s":
                startpos = int(raw_input("    secs? "))
            elif cmd == "r":
                break

        tn.write("stop\n")
        while True:
            print "  answer menu:"
            print "    a: track X = track A"
            print "    b: track X = track B"
            res = raw_input("    ? ")
            if res == "a":
                xid = 0
                break
            elif res == "b":
                xid = 1
                break
        
        if xid == trackx:
            correctnum += 1

    return correctnum

    
def main():
    print "mpcABX - super simple ABX client for MPD - by Marco Pontello"
    mpdaddr = raw_input("MPD address? ")
    mpdport = raw_input("MPD port? (6600) ")
    if not mpdport:
        mpdport = 6600
    trialsnum = raw_input("number of trials? (10) ")
    if not trialsnum:
        trialsnum = 10
    else:
        trialsnum = int(trialsnum)
    desc = raw_input("test description? ")
    
    logging.basicConfig(filename='mpcabx.log', filemode='w', level=logging.INFO,
                        format='%(asctime)s %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')
    logging.info("mpcABX - '%s'" % desc)
    correctnum = abx(mpdaddr, mpdport, trialsnum, "mpcabx.log")    

    print
    print "test concluded."
    print "correct answers: %i/%i" % (correctnum, trialsnum)
    logging.info("correct answers: %i/%i" % (correctnum, trialsnum))
    logging.info("probability of randomly guessing: %i%%" % guessprob(trialsnum, correctnum))

    
if __name__ == "__main__":
    main()
